#include "TwineCap.h"
#include "Arduino.h"

#define CAP_DDR      DDRB
#define CAP_PORT     PORTB
#define CAP_PIN      PINB
// AVR PIN NAMES
#define TOP_CLOUD_CAP_SEND 4
#define TOP_CLOUD_CAP_RECV 3
#define BOT_CLOUD_CAP_SEND 2
#define BOT_CLOUD_CAP_RECV 0

// Arduino pin names:
#define TOP_CLOUD_CAP_SEND_PIN 12
#define TOP_CLOUD_CAP_RECV_PIN 11
#define BOT_CLOUD_CAP_SEND_PIN 10
#define BOT_CLOUD_CAP_RECV_PIN 8

#ifdef __SAM3X8E__ 
#define COUNTER_TYPE uint32_t
#else 
#define COUNTER_TYPE uint16_t
#endif

int get_cap_lining(int lining);

enum {
UPPER_LINING = 0,
LOWER_LINING = 1
};

static COUNTER_TYPE _cap_timeout;
static int32_t _cap_thresh;
static COUNTER_TYPE _cap_calib[2];

void init_cap()
{
    int setupCapSense = true;
    //
    #ifdef __SAM3X8E__ 
    // we don't currently support capsense on Due
    // The arduino environment is still in beta and acting odd. 
     setupCapSense = false;
    #endif

    if(setupCapSense)
    {
        noInterrupts();
        _cap_thresh = 10;
#ifdef __SAM3X8E__ 
        // Just going to define it for now
        //_cap_timeout = (1000.0f * SystemCoreClock) / 16000000;
        _cap_timeout = 1000;
        // I am not sure if this these functions are as slow on the ATMEL
        // but the ARM is probably fast enough for it not to matter.
        pinMode(TOP_CLOUD_CAP_RECV_PIN, OUTPUT);
        pinMode(BOT_CLOUD_CAP_RECV_PIN, OUTPUT);

        digitalWrite(TOP_CLOUD_CAP_RECV_PIN, LOW);
        digitalWrite(BOT_CLOUD_CAP_RECV_PIN, LOW);

        pinMode(TOP_CLOUD_CAP_SEND_PIN, INPUT);
        pinMode(BOT_CLOUD_CAP_SEND_PIN, INPUT);
#else
        // Make our timeout portable across crystal types.
        _cap_timeout = (1000.0f * F_CPU) / 16000000;
        // initialize capacitive sensing receive ports to be low.
        CAP_DDR   &= ~(1<<TOP_CLOUD_CAP_RECV) & ~(1<<BOT_CLOUD_CAP_RECV);
        CAP_PORT  &= ~(1<<TOP_CLOUD_CAP_RECV) & ~(1<<BOT_CLOUD_CAP_RECV);
        
        CAP_DDR   |=  (1<<TOP_CLOUD_CAP_SEND) |  (1<<BOT_CLOUD_CAP_SEND);
#endif
        interrupts();
        
        // calibrate our capacitance
        
        int cycles = 8;

        COUNTER_TYPE avg_top = 0;
        COUNTER_TYPE avg_bot = 0;

        for(int i = 0; i < cycles; i++)
        {
             int temp_top_val = get_cap_lining(UPPER_LINING);
             if(temp_top_val > 0 && avg_top >= 0)
             {
                avg_top += temp_top_val;
             } else {
                avg_top = temp_top_val;
             }

             int temp_bot_val = get_cap_lining(LOWER_LINING);
             if(temp_bot_val > 0 && avg_bot >= 0)
             {
                avg_bot += temp_bot_val;
             } else {
                avg_bot = temp_bot_val;
             }
        }

        if( avg_top >= 0 )
        {
            _cap_calib[UPPER_LINING] = avg_top / cycles;
        }

        if( avg_bot >= 0 )
        {
            _cap_calib[LOWER_LINING] = avg_bot / cycles;
        }
    } else {
        _cap_timeout = 0;
    }
}

int get_cap_value(int sendBit, int recvBit)
{	
	noInterrupts();
    COUNTER_TYPE cap_total = 0;
#ifdef __SAM3X8E__ 
    digitalWrite(sendBit, LOW);
    pinMode(recvBit, INPUT);
    digitalWrite(recvBit, LOW);
    pinMode(recvBit, OUTPUT);
    pinMode(recvBit, INPUT);
    // Send goes high after we have cleared charge out of 
    // both sides of the resistor
    digitalWrite(sendBit, HIGH);
    while( (digitalRead(recvBit) != HIGH) && (cap_total < _cap_timeout ))
    {
		cap_total++;
	}
#else
    // Hold send low
	CAP_PORT  &= ~sendBit;
	// Recv input and no pull_up
	CAP_DDR   &= ~recvBit;
    CAP_PORT  &= ~recvBit;
    // Set pin output+low to pull recv line low.
	CAP_DDR   |= recvBit;
    // Switch back to input
    CAP_DDR   &= ~recvBit;
    // Pull the send high
	CAP_PORT  |= sendBit;
    // Wait until the recv goes high
	while( !(CAP_PIN & recvBit) && (cap_total < _cap_timeout ))
    {
		cap_total++;
	}
#endif    
    interrupts();
	
	if (cap_total >= _cap_timeout)
    {
		return -1;
    }
    return cap_total;
}

int get_cap_lining(int lining)
{
    int send_bit;
    int recv_bit;

    if(lining == UPPER_LINING)
    {
#if __SAM3X8E__
        send_bit = TOP_CLOUD_CAP_SEND_PIN;
        recv_bit = TOP_CLOUD_CAP_RECV_PIN;
#else 
        send_bit = 1<<TOP_CLOUD_CAP_SEND;
        recv_bit = 1<<TOP_CLOUD_CAP_RECV;
#endif
    } else {
#if __SAM3X8E__
        send_bit = BOT_CLOUD_CAP_SEND_PIN;
        recv_bit = BOT_CLOUD_CAP_RECV_PIN;
#else 
        send_bit = 1<<BOT_CLOUD_CAP_SEND;
        recv_bit = 1<<BOT_CLOUD_CAP_RECV;
#endif

    }
    return get_cap_value(send_bit, recv_bit);
}

int get_cap_lining_avg(int lining)
{
    int avg_val = 0;
    int cycles = 8;
    for(int i = 0; i < cycles; i++)
    {
         int temp_val = get_cap_lining(lining);
         if(temp_val > 0 && avg_val >= 0)
         {
            avg_val += temp_val;
         } else {
            avg_val = temp_val;
         }
    }

    avg_val = avg_val / cycles;

    if(avg_val < 0)
    {
        return avg_val;
    }

    if (avg_val < _cap_calib[lining])
    {
        return 0;
    } else {
        return avg_val - _cap_calib[lining];
    }
}

bool top_surface_touched()
{
    if(_cap_timeout == 0)
    {
        // capacitance wasn't setup
        return false;
    } else {
        int cap_val = get_cap_lining_avg(UPPER_LINING);
//        Serial.print("TOP: ");
//        Serial.println(cap_val);
        return cap_val > _cap_thresh || cap_val == -1;
    }
}

bool bot_surface_touched()
{
    if(_cap_timeout == 0)
    {
        // capacitance wasn't setup
        return false;
    } else {
        int cap_val = get_cap_lining_avg(LOWER_LINING);
//        Serial.print("BOT: ");
//        Serial.println(cap_val);
        return cap_val > _cap_thresh || cap_val == -1;
    }
}

